@@ -1,18 +1,13 @@
 from typing import Any, Dict, List, Optional, Set, Union, cast
 
-from openapi_schema_pydantic import OpenAPI, Schema
 from openapi_schema_pydantic.util import construct_open_api_with_schema_class
-from pydantic import Extra, validate_arguments
+from openapi_schema_pydantic.v3.v3_1_0.open_api import OpenAPI
+from pydantic import validate_arguments
 from pydantic.typing import AnyCallable
-from starlette.exceptions import HTTPException as StarletteHTTPException
-from starlette.middleware import Middleware
-from starlette.middleware.base import BaseHTTPMiddleware
+from starlette.middleware import Middleware as StarletteMiddleware
 from starlette.middleware.cors import CORSMiddleware
-from starlette.middleware.errors import ServerErrorMiddleware
 from starlette.middleware.trustedhost import TrustedHostMiddleware
-from starlette.responses import Response as StarletteResponse
 from starlette.staticfiles import StaticFiles
-from starlette.status import HTTP_500_INTERNAL_SERVER_ERROR
 from starlette.types import ASGIApp, Receive, Scope, Send
 from typing_extensions import Type
 
@@ -24,11 +19,12 @@ from starlite.config import (
     StaticFilesConfig,
     TemplateConfig,
 )
-from starlite.connection import Request
 from starlite.datastructures import State
-from starlite.enums import MediaType
-from starlite.exceptions import HTTPException
-from starlite.handlers import BaseRouteHandler, HTTPRouteHandler, asgi
+from starlite.handlers.asgi import ASGIRouteHandler, asgi
+from starlite.handlers.base import BaseRouteHandler
+from starlite.handlers.http import HTTPRouteHandler
+from starlite.handlers.websocket import WebsocketRouteHandler
+from starlite.middleware import ExceptionHandlerMiddleware
 from starlite.openapi.path_item import create_path_item
 from starlite.plugins.base import PluginProtocol
 from starlite.provide import Provide
@@ -36,7 +32,6 @@ from starlite.response import Response
 from starlite.router import Router
 from starlite.routes import ASGIRoute, BaseRoute, HTTPRoute, WebSocketRoute
 from starlite.signature import model_function_signature
-from starlite.template import TemplateEngineProtocol
 from starlite.types import (
     AfterRequestHandler,
     BeforeRequestHandler,
@@ -44,11 +39,11 @@ from starlite.types import (
     ExceptionHandler,
     Guard,
     LifeCycleHandler,
-    MiddlewareProtocol,
+    Middleware,
     ResponseHeader,
 )
 from starlite.utils import normalize_path
-from starlite.utils.exception import get_exception_handler
+from starlite.utils.templates import create_template_engine
 
 DEFAULT_OPENAPI_CONFIG = OpenAPIConfig(title="Starlite API", version="1.0.0")
 DEFAULT_CACHE_CONFIG = CacheConfig()
@@ -56,9 +51,9 @@ DEFAULT_CACHE_CONFIG = CacheConfig()
 
 class Starlite(Router):
     __slots__ = (
+        "asgi_handler",
         "asgi_router",
         "debug",
-        "middleware_stack",
         "openapi_schema",
         "plugins",
         "state",
@@ -67,6 +62,8 @@ class Starlite(Router):
         "plain_routes",
         "template_engine",
         "cache_config",
+        "cors_config",
+        "allowed_hosts",
         # the rest of __slots__ are defined in Router and should not be duplicated
         # see: https://stackoverflow.com/questions/472000/usage-of-slots
     )
@@ -75,29 +72,25 @@ class Starlite(Router):
     def __init__(
         self,
         *,
-        route_handlers: List[ControllerRouterHandler],
+        after_request: Optional[AfterRequestHandler] = None,
         allowed_hosts: Optional[List[str]] = None,
+        before_request: Optional[BeforeRequestHandler] = None,
+        cache_config: CacheConfig = DEFAULT_CACHE_CONFIG,
         cors_config: Optional[CORSConfig] = None,
         debug: bool = False,
         dependencies: Optional[Dict[str, Provide]] = None,
         exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
         guards: Optional[List[Guard]] = None,
-        middleware: Optional[List[Union[Middleware, Type[BaseHTTPMiddleware], Type[MiddlewareProtocol]]]] = None,
+        middleware: Optional[List[Middleware]] = None,
         on_shutdown: Optional[List[LifeCycleHandler]] = None,
         on_startup: Optional[List[LifeCycleHandler]] = None,
         openapi_config: Optional[OpenAPIConfig] = DEFAULT_OPENAPI_CONFIG,
+        plugins: Optional[List[PluginProtocol]] = None,
         response_class: Optional[Type[Response]] = None,
         response_headers: Optional[Dict[str, ResponseHeader]] = None,
-        plugins: Optional[List[PluginProtocol]] = None,
-        # connection-lifecycle hook handlers
-        before_request: Optional[BeforeRequestHandler] = None,
-        after_request: Optional[AfterRequestHandler] = None,
-        # static files
+        route_handlers: List[ControllerRouterHandler],
         static_files_config: Optional[Union[StaticFilesConfig, List[StaticFilesConfig]]] = None,
-        # template
         template_config: Optional[TemplateConfig] = None,
-        # cache
-        cache_config: CacheConfig = DEFAULT_CACHE_CONFIG,
     ):
         self.debug = debug
         self.state = State()
@@ -107,6 +100,9 @@ class Starlite(Router):
         self.static_paths = set()
         self.plain_routes: Set[str] = set()
         self.cache_config = cache_config
+        self.cors_config = cors_config
+        self.allowed_hosts = allowed_hosts
+
         super().__init__(
             dependencies=dependencies,
             guards=guards,
@@ -116,17 +112,16 @@ class Starlite(Router):
             route_handlers=route_handlers,
             before_request=before_request,
             after_request=after_request,
+            middleware=middleware,
         )
-        self.asgi_router = StarliteASGIRouter(on_shutdown=on_shutdown or [], on_startup=on_startup or [], app=self)
+
         self.exception_handlers: Dict[Union[int, Type[Exception]], ExceptionHandler] = exception_handlers or {}
-        self.middleware_stack: ASGIApp = self.build_middleware_stack(
-            user_middleware=middleware or [], cors_config=cors_config, allowed_hosts=allowed_hosts
-        )
+        self.asgi_router = StarliteASGIRouter(on_shutdown=on_shutdown or [], on_startup=on_startup or [], app=self)
+        self.asgi_handler = self.create_asgi_handler()
+        self.openapi_schema: Optional[OpenAPI] = None
         if openapi_config:
             self.openapi_schema = self.create_openapi_schema_model(openapi_config=openapi_config)
             self.register(openapi_config.openapi_controller)
-        else:
-            self.openapi_schema = None
         if static_files_config:
             for config in static_files_config if isinstance(static_files_config, list) else [static_files_config]:
                 path = normalize_path(config.path)
@@ -134,38 +129,44 @@ class Starlite(Router):
                 static_files = StaticFiles(html=config.html_mode, check_dir=False)
                 static_files.all_directories = config.directories  # type: ignore
                 self.register(asgi(path=path)(static_files))
-        if template_config:
-            self.template_engine: Optional[TemplateEngineProtocol] = template_config.engine(template_config.directory)
-        else:
-            self.template_engine = None
+        self.template_engine = create_template_engine(template_config)
+
+    def create_asgi_handler(self) -> ASGIApp:
+        """
+        Creates an ASGIApp that wraps the ASGI router inside an exception handler.
+
+        If CORS or TruseedHost configs are provided to the constructor, they will wrap the router as well.
+        """
+        asgi_handler: ASGIApp = self.asgi_router
+        if self.allowed_hosts:
+            asgi_handler = TrustedHostMiddleware(app=asgi_handler, allowed_hosts=self.allowed_hosts)
+        if self.cors_config:
+            asgi_handler = CORSMiddleware(app=asgi_handler, **self.cors_config.dict())
+        return self.wrap_in_exception_handler(
+            asgi_handler,
+            exception_handlers=self.exception_handlers,
+        )
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
+        """
+        The application entry point.
+        Lifespan events (startup / shutdown) are sent to the lifespan handler, otherwise the ASGI handler is used
+        """
         scope["app"] = self
         if scope["type"] == "lifespan":
             await self.asgi_router.lifespan(scope, receive, send)
             return
-        try:
-            scope["state"] = {}
-            await self.middleware_stack(scope, receive, send)
-        except Exception as e:  # pylint: disable=broad-except
-            await self.handle_exception(scope=scope, receive=receive, send=send, exc=e)
+        scope["state"] = {}
+        await self.asgi_handler(scope, receive, send)
 
-    async def handle_exception(self, scope: Scope, receive: Receive, send: Send, exc: Exception) -> None:
+    def wrap_in_exception_handler(
+        self, app: ASGIApp, exception_handlers: Dict[Union[int, Type[Exception]], ExceptionHandler]
+    ) -> ASGIApp:
         """
-        Determine what exception handler to use given the current scope, and handle an appropriate response
+        Wraps the given ASGIApp in an instance of ExceptionHandlerMiddleware
         """
-        status_code = exc.status_code if isinstance(exc, StarletteHTTPException) else HTTP_500_INTERNAL_SERVER_ERROR
-        if scope["type"] == "http":
-            exception_handler = (
-                get_exception_handler(self.exception_handlers, exc) or self.default_http_exception_handler
-            )
-            response = exception_handler(Request(scope=scope, receive=receive, send=send), exc)
-            await response(scope=scope, receive=receive, send=send)
-        else:
-            # The 4000+ code range for websockets is customizable, hence we simply add it to the http status code
-            status_code += 4000
-            reason = repr(exc)
-            await send({"type": "websocket.close", "code": status_code, "reason": reason})
+
+        return ExceptionHandlerMiddleware(app=app, exception_handlers=exception_handlers, debug=self.debug)
 
     def construct_route_map(self) -> None:  # noqa: C901
         """
@@ -190,23 +191,50 @@ class Starlite(Router):
                     cur = cast(Dict[str, Any], cur[component])
             else:
                 self.route_map[path] = {"_components": set()}
-                cur = self.route_map[path]
                 self.plain_routes.add(path)
-            if "_handlers" not in cur:
-                cur["_handlers"] = {}
-            if "_handler_types" not in cur:
-                cur["_handler_types"] = set()
+                cur = self.route_map[path]
+            if "_path_parameters" not in cur:
+                cur["_path_parameters"] = route.path_parameters
+            if "_asgi_handlers" not in cur:
+                cur["_asgi_handlers"] = {}
+            if "_is_asgi" not in cur:
+                cur["_is_asgi"] = False
             if path in self.static_paths:
                 cur["static_path"] = path
-            handler_types = cast(Set[str], cur["_handler_types"])
-            handler_types.add(route.scope_type.value)
-            handlers = cast(Dict[str, BaseRoute], cur["_handlers"])
+                cur["_is_asgi"] = True
+            asgi_handlers = cast(Dict[str, ASGIApp], cur["_asgi_handlers"])
             if isinstance(route, HTTPRoute):
-                handlers["http"] = route
+                for method, handler_mapping in route.route_handler_map.items():
+                    handler, _ = handler_mapping
+                    asgi_handlers[method] = self.build_route_middleware_stack(route, handler)
             elif isinstance(route, WebSocketRoute):
-                handlers["websocket"] = route
+                asgi_handlers["websocket"] = self.build_route_middleware_stack(route, route.route_handler)
+            elif isinstance(route, ASGIRoute):
+                asgi_handlers["asgi"] = self.build_route_middleware_stack(route, route.route_handler)
+                cur["_is_asgi"] = True
+
+    def build_route_middleware_stack(
+        self,
+        route: Union[HTTPRoute, WebSocketRoute, ASGIRoute],
+        route_handler: Union[HTTPRouteHandler, WebsocketRouteHandler, ASGIRouteHandler],
+    ) -> ASGIApp:
+        """Constructs a middleware stack that serves as the point of entry for each route"""
+
+        # we wrap the route.handle method in the ExceptionHandlerMiddleware
+        asgi_handler = self.wrap_in_exception_handler(
+            app=route.handle, exception_handlers=route_handler.resolve_exception_handlers()
+        )
+
+        for middleware in route_handler.resolve_middleware():
+            if isinstance(middleware, StarletteMiddleware):
+                asgi_handler = middleware.cls(app=asgi_handler, **middleware.options)
             else:
-                handlers["asgi"] = route
+                asgi_handler = middleware(app=asgi_handler)
+
+        # we wrap the entire stack again in ExceptionHandlerMiddleware
+        return self.wrap_in_exception_handler(
+            app=asgi_handler, exception_handlers=route_handler.resolve_exception_handlers()
+        )
 
     def register(self, value: ControllerRouterHandler) -> None:  # type: ignore[override]
         """
@@ -223,6 +251,7 @@ class Starlite(Router):
             for route_handler in route_handlers:
                 self.create_handler_signature_model(route_handler=route_handler)
                 route_handler.resolve_guards()
+                route_handler.resolve_middleware()
                 if isinstance(route_handler, HTTPRouteHandler):
                     route_handler.resolve_response_class()
                     route_handler.resolve_before_request()
@@ -251,49 +280,6 @@ class Starlite(Router):
                     provided_dependency_names=route_handler.dependency_name_set,
                 )
 
-    def build_middleware_stack(
-        self,
-        user_middleware: List[Union[Middleware, Type[BaseHTTPMiddleware], Type[MiddlewareProtocol]]],
-        allowed_hosts: Optional[List[str]],
-        cors_config: Optional[CORSConfig],
-    ) -> ASGIApp:
-        """
-        Builds the middleware stack by passing middlewares in a specific order
-        """
-        current_app: ASGIApp = self.asgi_router
-        # last added middleware will be on the top of stack and it will therefore be called first.
-        # we therefore need to reverse the middlewares to keep the call order according to
-        # the middlewares' list provided by the user
-        for middleware in reversed(user_middleware):
-            if isinstance(middleware, Middleware):
-                current_app = middleware.cls(app=current_app, **middleware.options)
-            else:
-                current_app = middleware(app=current_app)
-        if allowed_hosts:
-            current_app = TrustedHostMiddleware(app=current_app, allowed_hosts=allowed_hosts)
-        if cors_config:
-            current_app = CORSMiddleware(app=current_app, **cors_config.dict())
-        return current_app
-
-    def default_http_exception_handler(self, request: Request, exc: Exception) -> StarletteResponse:
-        """Default handler for exceptions subclassed from HTTPException"""
-        status_code = exc.status_code if isinstance(exc, StarletteHTTPException) else HTTP_500_INTERNAL_SERVER_ERROR
-        if status_code == HTTP_500_INTERNAL_SERVER_ERROR and self.debug:
-            # in debug mode, we just use the serve_middleware to create an HTML formatted response for us
-            server_middleware = ServerErrorMiddleware(app=self)
-            return server_middleware.debug_response(request=request, exc=exc)
-        if isinstance(exc, HTTPException):
-            content = {"detail": exc.detail, "extra": exc.extra, "status_code": exc.status_code}
-        elif isinstance(exc, StarletteHTTPException):
-            content = {"detail": exc.detail}
-        else:
-            content = {"detail": repr(exc)}
-        return Response(
-            media_type=MediaType.JSON,
-            content=content,
-            status_code=status_code,
-        )
-
     def create_openapi_schema_model(self, openapi_config: OpenAPIConfig) -> OpenAPI:
         """
         Updates the OpenAPI schema with all paths registered on the root router
@@ -309,7 +295,4 @@ class Starlite(Router):
                 openapi_schema.paths[route.path_format or "/"] = create_path_item(
                     route=route, create_examples=openapi_config.create_examples
                 )
-        # we have to monkey patch the "openapi-schema-pydantic" library, because it doesn't allow extra which causes
-        # failures with third party libs such as ormar.
-        Schema.Config.extra = Extra.ignore
-        return construct_open_api_with_schema_class(openapi_schema)
+        return cast(OpenAPI, construct_open_api_with_schema_class(openapi_schema))
