@@ -42,6 +42,7 @@ from starlite.types import (
     ExceptionHandler,
     Guard,
     Method,
+    Middleware,
     ResponseHeader,
 )
 from starlite.utils import is_async_callable
@@ -72,16 +73,14 @@ class HTTPRouteHandler(BaseRouteHandler):
         "resolved_before_request",
         "resolved_headers",
         "resolved_response_class",
-        "resolved_exception_handlers",
         "response_class",
         "response_description",
         "response_headers",
         "status_code",
         "summary",
+        "sync_to_thread",
         "tags",
         "template_name",
-        "sync_to_thread",
-        "exception_handlers",
     )
 
     @validate_arguments(config={"arbitrary_types_allowed": True})
@@ -102,6 +101,7 @@ class HTTPRouteHandler(BaseRouteHandler):
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
         exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        middleware: Optional[List[Middleware]] = None,
         # sync only
         sync_to_thread: bool = False,
         # OpenAPI related attributes
@@ -134,14 +134,20 @@ class HTTPRouteHandler(BaseRouteHandler):
             self.status_code = HTTP_204_NO_CONTENT
         else:
             self.status_code = HTTP_200_OK
-        super().__init__(path=path, dependencies=dependencies, guards=guards, opt=opt)
+        super().__init__(
+            path=path,
+            dependencies=dependencies,
+            guards=guards,
+            opt=opt,
+            middleware=middleware,
+            exception_handlers=exception_handlers,
+        )
         self.after_request = after_request
         self.before_request = before_request
         self.background_tasks = background_tasks
         self.media_type = media_type
         self.response_class = response_class
         self.response_headers = response_headers
-        self.exception_handlers = exception_handlers
         self.cache = cache
         self.cache_key_builder = cache_key_builder
         self.sync_to_thread = sync_to_thread
@@ -165,9 +171,6 @@ class HTTPRouteHandler(BaseRouteHandler):
         self.resolved_before_request: Union[
             Optional[BeforeRequestHandler], Type[BaseRouteHandler.empty]
         ] = BaseRouteHandler.empty
-        self.resolved_exception_handlers: Union[
-            Dict[Union[int, Type[Exception]], ExceptionHandler], Type[BaseRouteHandler.empty]
-        ] = BaseRouteHandler.empty
 
     def __call__(self, fn: AnyCallable) -> "HTTPRouteHandler":
         """
@@ -252,19 +255,6 @@ class HTTPRouteHandler(BaseRouteHandler):
                 self.resolved_after_request = self.resolved_after_request.__func__
         return cast(Optional[AfterRequestHandler], self.resolved_after_request)
 
-    def resolve_exception_handlers(self) -> Dict[Union[int, Type[Exception]], ExceptionHandler]:
-        """
-        Resolves the exception_handlers by starting from the route handler and moving up.
-
-        This method is memoized so the computation occurs only once.
-        """
-        if self.resolved_exception_handlers is BaseRouteHandler.empty:
-            exception_handlers: Dict[Union[int, Type[Exception]], ExceptionHandler] = {}
-            for layer in reversed(list(self.ownership_layers())):
-                exception_handlers = {**exception_handlers, **(layer.exception_handlers or {})}
-            self.resolved_exception_handlers = exception_handlers
-        return cast(Dict[Union[int, Type[Exception]], ExceptionHandler], self.resolved_exception_handlers)
-
     @property
     def http_methods(self) -> List[Method]:
         """
@@ -373,6 +363,7 @@ class get(HTTPRouteHandler):
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
         exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        middleware: Optional[List[Middleware]] = None,
         # sync only
         sync_to_thread: bool = False,
         # OpenAPI related attributes
@@ -413,6 +404,7 @@ class get(HTTPRouteHandler):
             tags=tags,
             sync_to_thread=sync_to_thread,
             exception_handlers=exception_handlers,
+            middleware=middleware,
         )
 
 
@@ -433,6 +425,7 @@ class post(HTTPRouteHandler):
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
         exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        middleware: Optional[List[Middleware]] = None,
         # sync only
         sync_to_thread: bool = False,
         # OpenAPI related attributes
@@ -473,6 +466,7 @@ class post(HTTPRouteHandler):
             tags=tags,
             sync_to_thread=sync_to_thread,
             exception_handlers=exception_handlers,
+            middleware=middleware,
         )
 
 
@@ -493,6 +487,7 @@ class put(HTTPRouteHandler):
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
         exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        middleware: Optional[List[Middleware]] = None,
         # sync only
         sync_to_thread: bool = False,
         # OpenAPI related attributes
@@ -533,6 +528,7 @@ class put(HTTPRouteHandler):
             tags=tags,
             sync_to_thread=sync_to_thread,
             exception_handlers=exception_handlers,
+            middleware=middleware,
         )
 
 
@@ -553,6 +549,7 @@ class patch(HTTPRouteHandler):
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
         exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        middleware: Optional[List[Middleware]] = None,
         # sync only
         sync_to_thread: bool = False,
         # OpenAPI related attributes
@@ -593,6 +590,7 @@ class patch(HTTPRouteHandler):
             tags=tags,
             sync_to_thread=sync_to_thread,
             exception_handlers=exception_handlers,
+            middleware=middleware,
         )
 
 
@@ -613,6 +611,7 @@ class delete(HTTPRouteHandler):
         cache: Union[bool, int] = False,
         cache_key_builder: Optional[CacheKeyBuilder] = None,
         exception_handlers: Optional[Dict[Union[int, Type[Exception]], ExceptionHandler]] = None,
+        middleware: Optional[List[Middleware]] = None,
         # sync only
         sync_to_thread: bool = False,
         # OpenAPI related attributes
@@ -653,4 +652,5 @@ class delete(HTTPRouteHandler):
             tags=tags,
             sync_to_thread=sync_to_thread,
             exception_handlers=exception_handlers,
+            middleware=middleware,
         )
